﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.PluginInfrastructure.Manager;
using System.IO;

namespace HeuristicLab.PluginInfrastructure.Advanced {
  public class InstallationManager {

    private string pluginDir;
    private PluginManager pluginManager;
    public InstallationManager() {
      this.pluginDir = Path.GetFullPath(HeuristicLab.PluginInfrastructure.Properties.Settings.Default.PluginDir);
      this.pluginManager = new PluginManager(pluginDir);
      this.pluginManager.DiscoverAndCheckPlugins();
    }

    public IEnumerable<string> Show(IEnumerable<string> pluginNames) {
      foreach (PluginDescription desc in GetPluginDescriptions(pluginNames)) {
        yield return ShowInformation(desc);
      }
    }

    private string ShowInformation(PluginDescription desc) {
      StringBuilder builder = new StringBuilder();
      builder.Append("Name: ").AppendLine(desc.Name);
      builder.Append("Version: ").AppendLine(desc.Version.ToString());
      builder.AppendLine("Description:").AppendLine(desc.Description);
      builder.Append("Build date: ").AppendLine(desc.BuildDate.ToString());
      builder.AppendLine("Files: ");
      foreach (string fileName in desc.Files) {
        builder.AppendLine(fileName);
      }
      builder.AppendLine("Directly depends on:");
      if (desc.Dependencies.Count() == 0) builder.AppendLine("None");
      foreach (var dependency in desc.Dependencies) {
        builder.AppendLine(dependency.Name);
      }
      builder.AppendFormat("Plugins directly dependent on {0}:\n", desc.Name);
      var dependents = from x in pluginManager.Plugins
                       where x.Dependencies.Contains(desc)
                       select x.Name;
      if (dependents.Count() == 0) builder.AppendLine("None");
      foreach (var dependent in dependents) {
        builder.AppendLine(dependent);
      }

      return builder.ToString();
    }

    private IEnumerable<PluginDescription> GetPluginDescriptions(IEnumerable<string> pluginNames) {
      return from name in pluginNames
             select (from pluginDesc in pluginManager.Plugins
                     where name == pluginDesc.Name
                     select pluginDesc).Single();
    }

    public void Install(IEnumerable<string> pluginNames) {
      throw new NotImplementedException();
    }

    public void Remove(IEnumerable<string> pluginNames) {
      foreach (PluginDescription pluginToDelete in PluginDescriptionIterator.IterateInDependencyOrder(GetPluginDescriptions(pluginNames))) {
        foreach (string fileName in pluginToDelete.Files) {
          Console.WriteLine("Deleting: " + Path.Combine(pluginDir, fileName));
        }
      }
    }

    public void Update(IEnumerable<string> pluginNames) {
      throw new NotImplementedException();
    }
  }
}
